﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Celeste;
using Celeste.Mod;
using Mono.Cecil.Cil;
using MonoMod.Cil;
using MonoMod.Utils;
using Microsoft.Xna.Framework;
using Monocle;



namespace BirdModStuff
{
    public class BirdModModule : EverestModule
    {
        public static BirdModModule Instance;
        public BirdModModule() { Instance = this; }

        public override Type SettingsType => typeof(BirdSettings);
        public static BirdSettings Settings => (BirdSettings)Instance._Settings;

        public static Dictionary<BirdSettings.BirdTypes, string> birdString = new Dictionary<BirdSettings.BirdTypes, string>
        {
            { BirdSettings.BirdTypes.Default, "bird" },
            { BirdSettings.BirdTypes.AfricanGrey, "ProBananaBirdMod_africangrey" },
            { BirdSettings.BirdTypes.CinnamonCockatiel, "ProBananaBirdMod_cockatiel" },
            { BirdSettings.BirdTypes.WhiteCockatoo, "ProBananaBirdMod_cockatoo" },
            { BirdSettings.BirdTypes.RedCrownedAmazon, "ProBananaBirdMod_amazon" },
            { BirdSettings.BirdTypes.GreenCheekedConure, "ProBananaBirdMod_conure" }
        };

        public static Dictionary<BirdSettings.BirdTypes, string> birdColor = new Dictionary<BirdSettings.BirdTypes, string>
        {
            {BirdSettings.BirdTypes.Default, "639bff" },
            {BirdSettings.BirdTypes.AfricanGrey, "808080" },
            {BirdSettings.BirdTypes.CinnamonCockatiel, "606060" },
            {BirdSettings.BirdTypes.WhiteCockatoo, "E5E5E5" },
            {BirdSettings.BirdTypes.RedCrownedAmazon, "4E7F1F" },
            {BirdSettings.BirdTypes.GreenCheekedConure, "BFBFBF" },
        };

        public override void Load()
        {
            On.Celeste.BirdNPC.ctor_Vector2_Modes += BirdNPCParticles;
            On.Celeste.FlingBird.ctor_Vector2Array_bool += FlingBirdParticles;
            IL.Celeste.BirdNPC.ctor_Vector2_Modes += stupidThing;
            IL.Celeste.FlingBird.ctor_Vector2Array_bool += BirdPath;
            IL.Celeste.BirdPath.ctor_EntityID_Vector2_Vector2Array_bool_bool_float += BirdPath;
            IL.Celeste.FlingBirdIntro.ctor_Vector2_Vector2Array_bool += stupidThing;
        }

        public override void Unload()
        {
            On.Celeste.BirdNPC.ctor_Vector2_Modes -= BirdNPCParticles;
            On.Celeste.FlingBird.ctor_Vector2Array_bool -= FlingBirdParticles;
            IL.Celeste.BirdNPC.ctor_Vector2_Modes -= stupidThing;
            IL.Celeste.FlingBird.ctor_Vector2Array_bool -= BirdPath;
            IL.Celeste.BirdPath.ctor_EntityID_Vector2_Vector2Array_bool_bool_float -= BirdPath;
            IL.Celeste.FlingBirdIntro.ctor_Vector2_Vector2Array_bool -= stupidThing;
        }

        private void BirdNPCParticles(On.Celeste.BirdNPC.orig_ctor_Vector2_Modes orig, BirdNPC self, Vector2 v, BirdNPC.Modes m)
        {
            orig(self, v, m);
            if (BirdNPC.P_Feather.Color != Calc.HexToColor(birdColor[Settings.BirdSkin])) BirdNPC.P_Feather.Color = Calc.HexToColor(birdColor[Settings.BirdSkin]);
        }

        private void FlingBirdParticles(On.Celeste.FlingBird.orig_ctor_Vector2Array_bool orig, FlingBird self, Vector2[] v, bool b)
        {
            orig(self, v, b);
            if(FlingBird.P_Feather.Color != Calc.HexToColor(birdColor[Settings.BirdSkin])) FlingBird.P_Feather.Color = Calc.HexToColor(birdColor[Settings.BirdSkin]);
        }

        //This hook works for all three cases. Nice. Technically I should use an IDetour for it (i think), but it really doesn't make that much difference.
        private void stupidThing(ILContext il)
        {
            ILCursor cursor = new ILCursor(il);
            while (cursor.TryGotoNext(MoveType.After, instr => instr.MatchLdstr("bird")))
            {
                cursor.EmitDelegate<Func<string, string>>(delegate(string _) { return birdString[Settings.BirdSkin]; }); //Delegate(string) pops the previous string off and "uses" it in the function
            }
        }

        private void BirdPath(ILContext il)
        {
            ILCursor cursor = new ILCursor(il);
            while(cursor.TryGotoNext(MoveType.After, instr => instr.MatchLdstr("639bff")))
            {
                cursor.EmitDelegate<Func<string, string>>(delegate (string s) { return birdColor[Settings.BirdSkin]; });
            }
            while (cursor.TryGotoNext(MoveType.After, instr => instr.MatchLdstr("bird")))
            {
                cursor.EmitDelegate<Func<string, string>>(delegate (string _) { return birdString[Settings.BirdSkin]; }); //Delegate(string) pops the previous string off and "uses" it in the function
            }
        }
    }

    public class BirdSettings : EverestModuleSettings
    {
        public enum BirdTypes
        {
            Default = 0,
            AfricanGrey = 1,
            CinnamonCockatiel = 2,
            WhiteCockatoo = 3,
            RedCrownedAmazon = 4,
            GreenCheekedConure = 5

        }

        
        public BirdTypes BirdSkin { get; set; } = BirdTypes.Default;
    }
}
